@@ -1,4 +1,5 @@
-DMR4lm_help <- function(X, y, clust.method = 'complete'){
+DMR4lm_help <- function(X, y, clust.method, lam){
+
     n <- nrow(X)
     nn <- sapply(1:ncol(X), function(i) class(X[,i]))
     names(nn) <- colnames(X)
@@ -17,9 +18,12 @@ DMR4lm_help <- function(X, y, clust.method = 'complete'){
     cont <- which(nn == "numeric")
     n.cont <- length(cont)
     namCont <- names(nn)[cont]
-	  #QR decompostion of the model matrix
-    qX <- qr.Q(m$qr)
-    rX <- qr.R(m$qr)
+                                #SzN there were cases (e.g. in Insurance dataset)
+                                #when the original columns
+                                #are lineary dependant
+                                #(case not excluded even after grpreg was run for execution paths from DMRnet)
+    qX <- qr.Q(m$qr, complete=FALSE) #SzN: explicitly stating that we want partial results (https://www.rdocumentation.org/packages/base/versions/3.6.2/topics/QR.Auxiliaries)
+    rX <- qr.R(m$qr) + diag(rep(lam, ncol(qX))) #SzN to solve the abovementioned matrix singularity we introduce the regularization of rX with a diagonal matrix
     Ro <- solve(rX)
     z <- t(qX)%*%y
     sigma <- as.numeric((t(m$res)%*%m$res)/(n - p))
